from django.conf.urls.defaults import patterns, url

urlpatterns = patterns(
    'likes.views',
    url(r'^like/(?P<content_type>[\w-]+)/(?P<id>\d+)/(?P<vote>-?\d+)$', 'like',
        name='like'),
)



import django.dispatch

likes_enabled_test = django.dispatch.Signal(
    providing_args=['instance', 'request']
)
can_vote_test = django.dispatch.Signal(
    providing_args=['instance', 'user', 'request']
)



from secretballot.middleware import SecretBallotIpUseragentMiddleware


class SecretBallotUserIpUseragentMiddleware(SecretBallotIpUseragentMiddleware):
    def generate_token(self, request):
        if request.user.is_authenticated():
            return request.user.username
        else:
            return super(SecretBallotUserIpUseragentMiddleware, self).\
                    generate_token(request)



from django.http import HttpResponseNotFound

from secretballot import views
from secretballot.models import Vote

from likes.utils import can_vote


def can_vote_test(request, content_type, object_id, vote):
    return can_vote(
        content_type.model_class().objects.get(id=object_id),
        request.user,
        request
    )


def like(request, content_type, id, vote):
    # Crawlers will follow the like link if anonymous liking is enabled. They
    # typically do not have referrer set.
    if 'HTTP_REFERER' not in request.META:
        return HttpResponseNotFound()

    url_friendly_content_type = content_type
    content_type = content_type.replace("-", ".")
    if request.is_ajax():
        return views.vote(
            request,
            content_type=content_type,
            object_id=id,
            vote=vote,
            template_name='likes/inclusion_tags/likes.html',
            can_vote_test=can_vote_test,
            extra_context={
                'likes_enabled': True,
                'can_vote': False,
                "content_type": url_friendly_content_type
            }
        )
    else:
        # Redirect to referer but append unique number(determined
        # from global vote count) to end of URL to bypass local cache.
        redirect_url = '%s?v=%s' % (request.META['HTTP_REFERER'], \
                Vote.objects.count() + 1)
        return views.vote(
            request,
            content_type=content_type,
            object_id=id,
            vote=vote,
            redirect_url=redirect_url,
            can_vote_test=can_vote_test
        )



from setuptools import setup, find_packages

setup(
    name='django-likes',
    version='0.0.8',
    description='Django app providing view interface to django-secretballot.',
    long_description = open('README.rst', 'r').read() + open('AUTHORS.rst', 'r').read() + open('CHANGELOG.rst', 'r').read(),
    author='Praekelt Foundation',
    author_email='dev@praekelt.com',
    license='BSD',
    url='http://github.com/praekelt/django-likes',
    packages = find_packages(),
    include_package_data=True,
    install_requires = [
        'django-secretballot',
    ],
    tests_require=[
        'django-setuptest>=0.0.6',
    ],
    test_suite="setuptest.setuptest.SetupTestSuite",
    classifiers = [
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)



from django.test import TestCase
from django.contrib.auth.models import User
from django.test.client import Client as BaseClient, FakePayload
from django.core.handlers.wsgi import WSGIRequest

import secretballot
from likes import middleware, models, urls, views
from likes.templatetags import likes_inclusion_tags


class Client(BaseClient):
    """Bug in django/test/client.py omits wsgi.input"""

    def _base_environ(self, **request):
        result = super(Client, self)._base_environ(**request)
        result['HTTP_USER_AGENT'] = 'Django Unittest'
        result['HTTP_REFERER'] = 'dummy'
        result['wsgi.input'] = FakePayload('')
        return result


class TestCase(TestCase):
    """Liking cannot be done programatically since it is tied too closely to a
    request. Do through-the-web tests."""

    def setUp(self):
        secretballot.enable_voting_on(User)
        self.user = User.objects.create_user(
            'john',
            'john@foo.com',
            'password'
        )
        self.client = Client()

    def test_like(self):
        # Anonymous vote
        response = self.client.get('/like/auth-user/%s/1' % self.user.id)
        # Expect a redirect
        self.assertEqual(response.status_code, 302)

    def test_like_ajax(self):
        # Anonymous vote
        response = self.client.get(
            '/like/auth-user/%s/1' % self.user.id,
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        self.assertEqual(response.status_code, 200)
